#include <qfile.h>
#include <qdatetime.h>
#include <qdir.h>
#include <qtextstream.h>
#include <qurl.h>
#include <qmessagebox.h>

#include "EventThread.h"
#include "WinampController.h"
#include "Settings.h"

char* ct() {
	time_t current = time(0);
	char* timeString = ctime(&current);
	timeString[strlen(timeString)-1]=0;
	return timeString;
}

EventThread::~EventThread() {
	QFile conf(QDir::homePath() + "/.winampspy/winampspy.config.txt");
	conf.open(QIODevice::WriteOnly);
	QTextStream stream(&conf);
	
	stream << configuration.login + "\n";
	stream << configuration.password + "\n";
	stream << (configuration.enabled ? "true\n" : "false\n");
	stream << configuration.host + "\n";
	stream << configuration.port + "\n";
	stream << configuration.path + "\n";
	stream << QString::number(configuration.minPacketSize) + "\n";
	stream.flush();
	conf.close();
	
	delete controller;
	delete timer;
	delete header;
	delete settings;
	#ifdef DEBUG
	fprintf(outf, "[DEBUG] [%s] EventThread::~EventThread() - end\n", ct());
	fclose(outf);
	#endif
}

void EventThread::showConfigDialog() {
	settings->show();
	settings->raise();
}
	
void EventThread::submit() {
	#ifdef DEBUG
	fprintf(outf, "[DEBUG] [%s] EventThread::submit()\n", ct());
	#endif
	
	QString xml;
	QDateTime time;
	time.setTime_t(currentTrack.startDate);
	
	xml = "<track><startDate>";
	xml += time.toString("dd.MM.yyyy hh:mm:ss");
	xml += "</startDate><album>";
	xml += currentTrack.album;
	xml += "</album><artist>";
	xml += currentTrack.artist;
	xml += "</artist><genre>";
	xml += currentTrack.genre;
	xml += "</genre><length>";
	QString mins = QString::number(currentTrack.length/60);
	QString secs = QString::number(currentTrack.length%60);
	if (currentTrack.length%60 < 10)
		secs = "0" + secs;
	xml += mins + ":" + secs;
	xml += "</length><title>";
	xml += currentTrack.title;
	xml += "</title><tracknumber>";
	xml += currentTrack.track;
	xml += "</tracknumber><year>";
	xml += currentTrack.year;
	xml += "</year></track>";
	
	save(xml);
}

void EventThread::save(QString xml) {
	QDir::home().mkdir(".winampspy");
	QFile songs(QDir::homePath() + "/.winampspy/songs.xml");
	QTextStream stream;
	
	if (!songs.exists())
		songs.open(QIODevice::WriteOnly);
	else
		songs.open(QIODevice::Append);
		
	stream.setDevice(&songs);
	
	stream << xml;
	stream << "\n";
	stream.flush();
	songs.close();
	
	QFileInfo info(songs);
	if (info.size() > configuration.minPacketSize * 1024)
		send();
}

void EventThread::send() {
	sending = true;

	QString xml = QString("<submission><id><login>%1</login><password>%2</password></id><tracks>").arg(configuration.login).arg(configuration.password);
	QFile songs(QDir::homePath() + "/.winampspy/songs.xml");
	songs.open(QIODevice::ReadOnly);
	QTextStream stream;
	stream.setDevice(&songs);
	xml += stream.readAll();
	songs.close();
	xml += "</tracks></submission>";
	QString searchString = "submission=" + QUrl::toPunycode(xml);
	client->request(*header, searchString.toUtf8());
}

EventThread::EventThread() : sending(false) {
	#ifdef DEBUG
	outf = fopen("C:\\log2.txt", "w");
	fprintf(outf, "[DEBUG] [%s] EventThread::EventThread() - start\n", ct());
	#endif	

	controller = new CWinampController();

	QFile conf(QDir::homePath() + "/.winampspy/winampspy.config.txt");
	conf.open(QIODevice::ReadWrite);
	QTextStream stream(&conf);

	configuration.login = stream.readLine();
	configuration.password = stream.readLine();
	QString enabled = stream.readLine();
	if (enabled == "true")
		configuration.enabled = true;
	else
		configuration.enabled = false;
	configuration.host = stream.readLine();
	configuration.port = stream.readLine();
	configuration.path = stream.readLine();
	configuration.minPacketSize = stream.readLine().toInt();

	conf.close();

	header = new QHttpRequestHeader("POST", configuration.path) ;
	header->setValue("Host", configuration.host);
	header->setContentType("application/x-www-form-urlencoded");
	
	settings = new SettingsDialog(&configuration);
	timer = new QTimer(this);
	connect(timer, SIGNAL(timeout()), this, SLOT(timeout()));
	timer->start(1000);
		client = new QHttp(configuration.host, configuration.port.toInt());
	
	connect(client, SIGNAL(requestFinished(int, bool)), this, SLOT(requestFinished(int, bool)));
	
	badPasswd = new BadPasswordDialog;

	#ifdef DEBUG
	fprintf(outf, "[DEBUG] [%s] EventThread::EventThread() - end\n", ct());
	#endif	
}

void EventThread::timeout() {
	if (!configuration.enabled)
		return;
	
	#ifdef DEBUG
	fflush(outf);
	fprintf(outf, "[DEBUG] [%s] EventThread::run() - tick\n", ct());
	fprintf(outf, "[DEBUG] [%s] EventThread::run() - now playing: %s [%d:%02d/%d:%02d]\n", ct(),
			controller->GetCurrentSongMetadata(PlayerController::META_TITLE).toAscii().constData(),
			controller->GetPlayingTime()/1000/60,
			controller->GetPlayingTime()/1000%60,
			controller->GetCurrentSongLength()/60,
			controller->GetCurrentSongLength()%60
		);
	#endif
	
	QString currentArtist = controller->GetCurrentSongMetadata(PlayerController::META_ARTIST);
	QString currentSong = controller->GetCurrentSongMetadata(PlayerController::META_TITLE);
	
	if (currentArtist != currentTrack.artist || currentSong != currentTrack.title) {
		// new song started
		currentTrack.length = controller->GetCurrentSongLength();
		if (currentTrack.length < 1)
			return;
		currentTrack.album = controller->GetCurrentSongMetadata(PlayerController::META_ALBUM);
		currentTrack.artist = controller->GetCurrentSongMetadata(PlayerController::META_ARTIST);
		currentTrack.genre = controller->GetCurrentSongMetadata(PlayerController::META_GENRE);
		currentTrack.title = controller->GetCurrentSongMetadata(PlayerController::META_TITLE);
		currentTrack.track = controller->GetCurrentSongMetadata(PlayerController::META_TRACK);
		currentTrack.year = controller->GetCurrentSongMetadata(PlayerController::META_YEAR);
		currentTrack.startDate = time(0);
		currentTrack.continousPlay = 0;
		currentTrack.submited = false;
	} else {
		// song continues to play
		int pos = controller->GetPlayingTime()/1000;
		time_t currentTime = time(0);
		
		if (currentTrack.startDate + pos < currentTime + 1 && currentTrack.startDate + pos > currentTime - 1) {
			// song was not rewinded
			currentTrack.continousPlay++;
		} else {
			// track was rewinded, reset continous play period
			currentTrack.continousPlay = 0;
		}
		
		// check if song is ready to submit
		if (!currentTrack.submited && currentTrack.continousPlay >= currentTrack.length/2 && !sending) {
			// submit the song
			currentTrack.submited = true;
			submit();
		}
	}
}

void EventThread::requestFinished(int, bool) {
	QHttpResponseHeader response = client->lastResponse();

	if (!response.isValid());
	else if (response.statusCode() == 200 || response.statusCode() == 400) {
		// sent succescfull or corrupted data - need to remove xml anyway
		QFile conf(QDir::homePath() + "/.winampspy/songs.xml");
		conf.open(QIODevice::WriteOnly);
		conf.close();
	} else if (response.statusCode() == 401)
		// unsuccesfull authorization
		badPasswd->show();
		

	sending = false;
}
