// WinampController.cpp: implementation of the CWinampController class.
//
//////////////////////////////////////////////////////////////////////

#include "WinampController.h"
#include <shellapi.h>
#include <stdio.h>

//////////////////////////////////////////////////////////////////////
// General Stuff
//////////////////////////////////////////////////////////////////////

CWinampController::CWinampController()
{
	m_hWndWinamp		= NULL;
	m_hWndWinampEQ		= NULL;
	m_hWndWinampPE		= NULL;
	m_hWndWinampMB		= NULL;
	m_strVersionString	= new char[32];		// Winamp Version String
	m_strWinampDir		= new char[1];		// Winamp Directory
	m_strWinampDir[0] = 0;
	
	Init(0);
}

CWinampController::~CWinampController()
{
	delete m_strVersionString;
	delete m_strWinampDir;
}

BOOL CWinampController::Init(BOOL bStartWinampIfNotRunning)
{	
	m_hWndWinamp = FindWindow("Winamp v1.x", NULL);

	if(m_hWndWinamp == NULL)
	{
		if(bStartWinampIfNotRunning)
		{
			::ShellExecute(NULL, "open", "c:\\progra~1\\winamp\\winamp.exe", NULL, "c:\\progra~1\\winamp\\", SW_MINIMIZE);

			//Give it time to show itself
			Sleep(1000);
			m_hWndWinamp = FindWindow("Winamp v1.x", NULL);

			if(m_hWndWinamp == NULL)
				return FALSE;
		}
	}
	
	m_hWndWinampEQ = FindWindow("Winamp EQ", NULL);
	m_hWndWinampPE = FindWindow("Winamp PE", NULL);
	m_hWndWinampMB = FindWindow("Winamp MB", NULL);

	GetWinampDir();
	return TRUE;
}

BOOL CWinampController::WindowsReady()
{
	BOOL bReady = TRUE;

	if(m_hWndWinamp == NULL || !IsWindow(m_hWndWinamp))
		bReady = FALSE;

	if(m_hWndWinampEQ == NULL || !IsWindow(m_hWndWinampEQ))
		bReady = FALSE;

	if(m_hWndWinampPE == NULL || !IsWindow(m_hWndWinampPE))
		bReady = FALSE;

	if(m_hWndWinampMB == NULL || !IsWindow(m_hWndWinampMB))
		bReady = FALSE;

	if(!bReady)
		Init(TRUE);

	return TRUE;
}

void CWinampController::SendCommand(long Code)
{
	
	SendMessage(m_hWndWinamp, WM_COMMAND, Code, 0);
}


//////////////////////////////////////////////////////////////////////
// General Winamp Stuff
//////////////////////////////////////////////////////////////////////

//Returned in Milliseconds
int	CWinampController::GetPlayingTime()
{
	return SendMessage(m_hWndWinamp,WM_WA_IPC,0,IPC_GETOUTPUTTIME);
}

long CWinampController::GetCurrentSongLength()
{
	
	return SendMessage(m_hWndWinamp,WM_WA_IPC, 1,IPC_GETOUTPUTTIME);
}

// Only works for Winamp 2.90 upwards
QString CWinampController::GetCurrentSongMetadata(QString MetaData)
{
	static char buffer[1024];
    char Filename[MAX_PATH];
    extendedFileInfoStruct  Info;

    memset(buffer, 0, sizeof(buffer));
    strcpy(Filename, GetCurrentSongFileName());

    Info.ret = buffer;
    Info.retlen = sizeof(buffer);
    Info.filename = Filename;
    Info.metadata = MetaData.toAscii().data();

  	SendMessage(m_hWndWinamp,WM_WA_IPC, (WPARAM)&Info, IPC_GET_EXTENDED_FILE_INFO);

    return QString(buffer);
}

char* CWinampController::GetCurrentSongFileName()
{
	return GetSongFileName(GetPlayListPos());
}

char* CWinampController::GetSongFileName(long PlayListPos)
{	
	return (char *)SendMessage(m_hWndWinamp,WM_WA_IPC,PlayListPos,IPC_GETPLAYLISTFILE);
}

long CWinampController::GetPlayListPos()
{
	return SendMessage(m_hWndWinamp,WM_WA_IPC,0,IPC_GETLISTPOS);
}

char* CWinampController::GetWinampDir()
{
	if(strlen(m_strWinampDir))
		return m_strWinampDir;

	char filename[512];
	char* Path;
	GetModuleFileName(NULL,filename,sizeof(filename));
	Path = filename;

	int pos;
	for (pos=strlen(Path)-1; pos>=0; pos--)
		if (Path[pos]=='/')
			break;
			
	if(pos > 0)
	{
		// Get Winamp Directory
		delete m_strWinampDir;
		m_strWinampDir = new char[strlen(Path)];
		strcpy(m_strWinampDir, Path);
		m_strWinampDir[pos] = 0;
		return m_strWinampDir;
	}
	
	return "";
}




/*
long CWinampController::GetVersion()
{
	return SendMessage(m_hWndWinamp,WM_WA_IPC,0,IPC_GETVERSION);
}

long CWinampController::GetStatus()
{
	return SendMessage(m_hWndWinamp,WM_WA_IPC,0,IPC_ISPLAYING);
}

void CWinampController::SetStatus(long newVal)
{
	
	switch(newVal)
	{
	case WINAMP_STOPPED:
		SendCommand(WINAMP_STOP);
		break;
	case WINAMP_PAUSED:
		SendCommand(WINAMP_PAUSE);
		break;
	default:
		Play();
		break;
	}
}


void CWinampController::NextTrack()
{
	SendCommand(WINAMP_FASTFWD);
}

void CWinampController::PrevTrack()
{
	SendCommand(WINAMP_REWIND);
}

long CWinampController::GetCurrentPos()
{
	
	return SendMessage(m_hWndWinamp,WM_WA_IPC,0,IPC_GETOUTPUTTIME);
}

void CWinampController::SetCurrentPos(long newVal)
{
	
	SendMessage(m_hWndWinamp,WM_WA_IPC,newVal,IPC_JUMPTOTIME);
}

void CWinampController::SetVolume(long newVal)
{
	
	if(newVal < 0)
	{
		newVal = 0;
	}
	if(newVal > 255)
	{
		newVal = 255;
	}
}

void CWinampController::IncreaseVolume()
{
	SendMessage(m_hWndWinamp, WM_COMMAND, WINAMP_VOLUMEUP, 0);
}

void CWinampController::DecreaseVolume()
{
	
	SendMessage(m_hWndWinamp, WM_COMMAND, WINAMP_VOLUMEDOWN, 0);
}

long CWinampController::GetSampleRate()
{
	
	return SendMessage(m_hWndWinamp,WM_WA_IPC,0,IPC_GETINFO);
}

long CWinampController::GetBitrate()
{
	
	return SendMessage(m_hWndWinamp,WM_WA_IPC,1,IPC_GETINFO);
}

long CWinampController::GetChannels()
{
	
	return SendMessage(m_hWndWinamp,WM_WA_IPC,2,IPC_GETINFO);
}

void CWinampController::SetPanning(long PanIndex)
{
	
	SendMessage(m_hWndWinamp,WM_WA_IPC,PanIndex,IPC_SETPANNING);
}

void CWinampController::ChangeDirectory(char* NewDir)
{
	if(strlen(NewDir))
	{
		COPYDATASTRUCT cds;
		cds.dwData = IPC_CHDIR;
		cds.lpData = (void *) NewDir;
		cds.cbData = strlen(NewDir); // include space for null char
		SendMessage(m_hWndWinamp,WM_COPYDATA,(WPARAM)NULL,(LPARAM)&cds);
	}
}

char* CWinampController::GetSongFileName(long PlayListPos)
{	
	char* Name;
	Name = (char *)SendMessage(m_hWndWinamp,WM_WA_IPC,PlayListPos,IPC_GETPLAYLISTFILE);
	
	return Name;
}

char* CWinampController::GetSongTitle(long PlayListPos)
{
	char* Name = (char *)SendMessage(m_hWndWinamp,WM_WA_IPC,PlayListPos,IPC_GETPLAYLISTTITLE);
	return Name;
}



char* CWinampController::GetCurrentSongTitle()
{
	return GetSongTitle(GetPlayListPos());
}

char* CWinampController::GetSkinName()
{
	static char Temp[256];
	SendMessage(m_hWndWinamp,WM_WA_IPC,(WPARAM)Temp,IPC_GETSKIN);
	return Temp;
}

void CWinampController::SetSkinName(char* Name)
{
	SendMessage(m_hWndWinamp,WM_WA_IPC,(WPARAM)Name,IPC_SETSKIN);	
}

void CWinampController::StartPlugIn(char* PlugInName)
{
	SendMessage(m_hWndWinamp,WM_WA_IPC,(WPARAM)PlugInName,IPC_EXECPLUG);
}

void CWinampController::ToggleShufflePlay()
{
	
	SendMessage(m_hWndWinamp, WM_COMMAND, WINAMP_FILE_SHUFFLE, 0);
}

void CWinampController::ToggleRepeatPlay()
{
	
	SendMessage(m_hWndWinamp, WM_COMMAND, WINAMP_FILE_REPEAT, 0);
}

void CWinampController::Stop()
{
	
	SendMessage(m_hWndWinamp, WM_COMMAND, WINAMP_STOP, 0);
}

void CWinampController::Pause()
{
	
	SendMessage(m_hWndWinamp, WM_COMMAND, WINAMP_PAUSE, 0);
}

void CWinampController::FastForward()
{
	
	SendMessage(m_hWndWinamp, WM_COMMAND, WINAMP_FASTFWD, 0);
}

void CWinampController::Rewind()
{
	
	SendMessage(m_hWndWinamp, WM_COMMAND, WINAMP_REWIND, 0);
}

void CWinampController::CloseWinamp()
{
	
	SendMessage(m_hWndWinamp, WM_CLOSE, 0, 0);
}

void CWinampController::SongChange()
{
}



void CWinampController::FastFwd5Secs()
{
	
	SendMessage(m_hWndWinamp, WM_COMMAND, WINAMP_FFWD5S, 0);
}

void CWinampController::Rewind5Secs()
{
	
	SendMessage(m_hWndWinamp, WM_COMMAND, WINAMP_REW5S, 0);
}

BOOL CWinampController::GetWinampVisible()
{
	return IsWindowVisible(m_hWndWinamp);
}

void CWinampController::SetWinampVisible(BOOL newVal)
{
	if(newVal)
	{
		if(!IsWindowVisible(m_hWndWinamp))
		{
			ShowWindow(m_hWndWinamp, SW_SHOW);
		}
	}
	else
	{
		if(IsWindowVisible(m_hWndWinamp))
		{
			ShowWindow(m_hWndWinamp, SW_HIDE);
		}
	}
}

BOOL CWinampController::GetShuffleStatus()
{
	
	if(GetMenuState(GetOptionMenu(), WINAMP_MENU_SHUFFLE, MF_BYCOMMAND) == MF_CHECKED)
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

void CWinampController::SetShuffleStatus(BOOL newVal)
{
	
	BOOL CurVal = GetShuffleStatus();
	if(CurVal != newVal)
	{
		ToggleShufflePlay();
	}
}

BOOL CWinampController::GetRepeatStatus()
{
	
	if(GetMenuState(GetOptionMenu(), WINAMP_MENU_REPEAT, MF_BYCOMMAND) == MF_CHECKED)
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

void CWinampController::SetRepeatStatus(BOOL newVal)
{
	
	BOOL CurVal = GetRepeatStatus();
	if(CurVal != newVal)
	{
		ToggleRepeatPlay();
	}
}

HMENU CWinampController::GetOptionMenu()
{
	
	// Get System menu handle
	HMENU hMenuSystem = GetSystemMenu(m_hWndWinamp, 0);

	//Get Winamp sub-menu handle
	HMENU hMenuWinamp = GetSubMenu(hMenuSystem, 0);

	//Get Options sub-menu handle
	return GetSubMenu(hMenuWinamp, WINAMP_MENU_OPTIONS_POS);
}

//////////////////////////////////////////////////////////////////////
// Playlist Stuff
//////////////////////////////////////////////////////////////////////

void CWinampController::ClearPlaylist()
{
	SendMessage(m_hWndWinamp,WM_WA_IPC,0,IPC_DELETE);
}



void CWinampController::SetPlayListPos(long newVal)
{
	SendMessage(m_hWndWinamp,WM_WA_IPC,newVal,IPC_SETPLAYLISTPOS);
}

long CWinampController::GetPlayListCount()
{
	return SendMessage(m_hWndWinamp,WM_WA_IPC,0,IPC_GETLISTLENGTH);
}

BOOL CWinampController::GetPlaylistVisible()
{
	return IsWindowVisible(m_hWndWinampPE);
}

void CWinampController::SetPlaylistVisible(BOOL newVal)
{
	
	if(newVal)
	{
		if(!IsWindowVisible(m_hWndWinampPE))
		{
			ShowWindow(m_hWndWinampPE, SW_SHOW);
		}
	}
	else
	{
		if(IsWindowVisible(m_hWndWinampPE))
		{
			ShowWindow(m_hWndWinampPE, SW_HIDE);
		}
	}
}

void CWinampController::DeletePEItem(int Pos)
{
	MovePEHighlight(Pos);
	DeletePECurHighlight();
}

void CWinampController::DeletePECurHighlight()
{
	SendMessage(m_hWndWinampPE, WM_COMMAND, WINAMP_DEL_FRM_PE, 0 );
}

void CWinampController::MovePEHighlight(int Pos)
{
	int PELength = GetPlayListCount();

	if(Pos < PELength)
	{
		SendMessage(m_hWndWinampPE, WM_COMMAND, WINAMP_MV_PE_HLT_UP, 0);

		// Move the selector down to the song to remove
		for(int i = 0; i < Pos; i++ )
			SendMessage(m_hWndWinampPE, WM_COMMAND, WINAMP_MV_PE_HLT_DOWN, 0 );
	}
}

bool CWinampController::MovePEItem(int curPos, int NewPos)
{
	int diff = 0;

	if(curPos == NewPos)
		return false;

	MovePEHighlight(curPos);

	if(curPos > NewPos)
	{
		diff = curPos - NewPos;

		for(int i = 0; i < diff; i++)
			SendMessage(m_hWndWinampPE, WM_COMMAND, WINAMP_MV_PE_ITEM_UP, 0 );
	}
	else
	{
		diff = NewPos - curPos;

		for(int i = 0; i < diff; i++)
			SendMessage(m_hWndWinampPE, WM_COMMAND, WINAMP_MV_PE_ITEM_DOWN, 0 );
	}

	return true;
}

typedef struct 
{ 
	char file[MAX_PATH]; 
	int index; 
} fileinfo; 
 
void CWinampController::PLInsertTrack(char* strFile, int nPos)
{
	COPYDATASTRUCT cds; 
	fileinfo f; 
	cds.dwData = IPC_PE_INSERTFILENAME; 
	strcpy(f.file, strFile); // Path to the new entry 
	f.index=nPos;                 // Position to insert at 
	cds.lpData = (void *)&f; 
	cds.cbData = sizeof(fileinfo); 
	::SendMessage(m_hWndWinampPE,WM_COPYDATA,(WPARAM)NULL,(LPARAM)&cds);
}

//////////////////////////////////////////////////////////////////////
// EQ Stuff
//////////////////////////////////////////////////////////////////////
long CWinampController::GetEqPosition(long Band)
{
	
	if(Band < 0)
	{
		Band = 0;
	}
	if(Band > 9)
	{
		Band = 9;
	}
	return SendMessage(m_hWndWinamp,WM_WA_IPC,Band,IPC_GETEQDATA);
}

void CWinampController::SetEqPosition(long Band, long newVal)
{
	
	if(Band < 0)
	{
		Band = 0;
	}
	if(Band > 9)
	{
		Band = 9;
	}
	if(newVal < 0)
	{
		newVal = 0;
	}
	if(newVal > 63)
	{
		newVal = 63;
	}
	SendMessage(m_hWndWinamp,WM_WA_IPC,Band,IPC_GETEQDATA);
	SendMessage(m_hWndWinamp,WM_WA_IPC,newVal,IPC_SETEQDATA);
}

long CWinampController::GetPreAmpPosition()
{
	
	return SendMessage(m_hWndWinamp,WM_WA_IPC,10,IPC_GETEQDATA);
	
}

void CWinampController::SetPreAmpPosition(long newVal)
{
	
	if(newVal < 0)
	{
		newVal = 0;
	}
	if(newVal > 63)
	{
		newVal = 63;
	}
	SendMessage(m_hWndWinamp,WM_WA_IPC,10,IPC_GETEQDATA);
	SendMessage(m_hWndWinamp,WM_WA_IPC,newVal,IPC_SETEQDATA);
}

BOOL CWinampController::GetEqEnabled()
{
	
	return SendMessage(m_hWndWinamp,WM_WA_IPC,11,IPC_GETEQDATA);
}

void CWinampController::SetEqEnabled(BOOL newVal)
{
	
	SendMessage(m_hWndWinamp,WM_WA_IPC,11,IPC_GETEQDATA);
	SendMessage(m_hWndWinamp,WM_WA_IPC,newVal,IPC_SETEQDATA);
}

BOOL CWinampController::GetAutoloadEnabled()
{
	
	return SendMessage(m_hWndWinamp,WM_WA_IPC,12,IPC_GETEQDATA);
}

void CWinampController::SetAutoloadEnabled(BOOL newVal)
{
	
	SendMessage(m_hWndWinamp,WM_WA_IPC,12,IPC_GETEQDATA);
	SendMessage(m_hWndWinamp,WM_WA_IPC,newVal,IPC_SETEQDATA);
}
BOOL CWinampController::GetEqualizerVisible()
{
	
	return IsWindowVisible(m_hWndWinampEQ);
}

void CWinampController::SetEqualizerVisible(BOOL newVal)
{
	
	if(newVal)
	{
		if(!IsWindowVisible(m_hWndWinampEQ))
		{
			ShowWindow(m_hWndWinampEQ, SW_SHOW);
		}
	}
	else
	{
		if(IsWindowVisible(m_hWndWinampEQ))
		{
			ShowWindow(m_hWndWinampEQ, SW_HIDE);
		}
	}
}

//////////////////////////////////////////////////////////////////////
// MB Stuff
//////////////////////////////////////////////////////////////////////

BOOL CWinampController::GetMiniBrowserVisible()
{
	
	return IsWindowVisible(m_hWndWinampMB);
}

void CWinampController::SetMiniBrowserVisible(BOOL newVal)
{
	
	if(newVal)
	{
		if(!IsWindowVisible(m_hWndWinampMB))
		{
			ShowWindow(m_hWndWinampMB, SW_SHOW);
		}
	}
	else
	{
		if(IsWindowVisible(m_hWndWinampMB))
		{
			ShowWindow(m_hWndWinampMB, SW_HIDE);
		}
	}
}

void CWinampController::SetMiniBrowserURL(char* strUrl, BOOL bForceOpen)
{
	char* lpszUrl = NULL;
	
	// Force open the mini-browser window
	if(bForceOpen)
		::SendMessage(m_hWndWinamp, WM_WA_IPC, NULL, IPC_MBOPEN);
	
	//Build the url string to be displayed
	lpszUrl = new char[strlen(strUrl)+1];
	
	sprintf(lpszUrl, "%s", strUrl);
	
	//Make the mini-browser navigate to the URL 
	::SendMessage(m_hWndWinamp,WM_WA_IPC, reinterpret_cast<WPARAM>(lpszUrl), IPC_MBOPENREAL);
	
	delete[] lpszUrl;
}

BOOL CWinampController::IsInetAvailable()
{
	return ::SendMessage(m_hWndWinamp,WM_WA_IPC,0,IPC_INETAVAILABLE);
}

void CWinampController::JumpToTrack(unsigned int Number)
{
	
	PostMessage(m_hWndWinamp,WM_COMMAND,WINAMP_BUTTON4,0);
	Number--;
	PostMessage(m_hWndWinamp,WM_WA_IPC,Number,IPC_SETPLAYLISTPOS);
	PostMessage(m_hWndWinamp,WM_COMMAND,WINAMP_BUTTON2,0);
}

char* CWinampController::GetWinampDir()
{
	if(strlen(m_strWinampDir))
		return m_strWinampDir;

	char filename[512];
	char* Path;
	GetModuleFileName(NULL,filename,sizeof(filename));
	Path = filename;

	int pos;
	for (pos=strlen(Path)-1; pos>=0; pos--)
		if (Path[pos]=='/')
			break;
			
	if(pos > 0)
	{
		// Get Winamp Directory
		delete m_strWinampDir;
		m_strWinampDir = new char[strlen(Path)];
		strcpy(m_strWinampDir, Path);
		m_strWinampDir[pos] = 0;
		return m_strWinampDir;
	}
	
	return "";
}

void CWinampController::SetVolume(int nVolume)
{
	if(nVolume < 0 || nVolume > 255)
		return;

	SendMessage(m_hWndWinamp,WM_WA_IPC,nVolume,IPC_SETVOLUME);
}

BOOL CWinampController::Enqueue(char* strFileName)
{
	COPYDATASTRUCT cds;
	cds.dwData = IPC_PLAYFILE;
	cds.lpData = (void *)strFileName;
	cds.cbData = strlen((char *) cds.lpData) + 1; // include space for null char

	::SendMessage(m_hWndWinamp, WM_COPYDATA,1,(LPARAM)&cds);

	return TRUE;
}

BOOL CWinampController::Play(char* strFileName, BOOL bKeepList)
{
	if(strlen(strFileName))
	{
		if(!bKeepList)
		{
			ClearPlaylist();
			Enqueue(strFileName);
		}
		else
		{
			PLInsertTrack(strFileName, 0);
		}
	}

	::SendMessage(m_hWndWinamp, WM_COMMAND,WINAMP_BUTTON4, 0);
	::SendMessage(m_hWndWinamp, WM_WA_IPC,0,IPC_STARTPLAY);

	return TRUE;
}

char* CWinampController::GetFileNameFromPos(int nPos)
{
	// What are we playing?
	char* Text;
	char buffer[MAX_PATH];
	memset(buffer, 0, MAX_PATH);
	char* p = buffer;
	p = reinterpret_cast<char*>(SendMessage(m_hWndWinamp,WM_WA_IPC,nPos,IPC_GETPLAYLISTFILE));
	Text = p;
	return Text;
}

BOOL CWinampController::IsPlaying()
{
	return (GetWinampState() == WA_STATE_PLAY);
}

BOOL CWinampController::IsPaused()
{
	return (GetWinampState() == WA_STATE_PAUSE);
}

// Returned in SECONDS
int	CWinampController::GetTrackLength()
{
	int nTrackLength = SendMessage(m_hWndWinamp,WM_WA_IPC,1,IPC_GETOUTPUTTIME);
	
	return nTrackLength;
}



char* CWinampController::GetCurrTrackPath()
{
	int nPos = SendMessage(m_hWndWinamp,WM_WA_IPC,0,IPC_GETLISTPOS);
	return GetFileNameFromPos(nPos);
}

int CWinampController::GetWinampState()
{
	// Are actually playing?
	int ret = SendMessage(m_hWndWinamp,WM_WA_IPC,0,IPC_ISPLAYING);
	if(ret == 1)
		return WA_STATE_PLAY;
	else if(ret == 3)
		return WA_STATE_PAUSE;
	else if(ret == 0)
		return WA_STATE_STOP;

	return -1;
}

BOOL CWinampController::IsPEWindowWinshade()
{
	RECT rcPEWnd;
	
	::GetWindowRect(m_hWndWinampPE, &rcPEWnd);

	if((rcPEWnd.bottom - rcPEWnd.top) < 116)
		return TRUE;
	
	return FALSE;
}

int CWinampController::GetVolume()
{
	return -1;
}
*/


