/*##############################################
##
## Module				: CWinampController
## Description	: This class takes total control
##								of Winamp 1.x - 2.x
##
## Author(s)		: Spib
##
## Modifications
##
##############################################*/

#pragma once

#include <qstring.h>

#include "Gen.h"
#include "wa_ipc.h"
#include "WinampCmd.h"
#include "PlayerController.h"

#define WINAMP_MENU_OPTIONS_POS		11
#define WINAMP_MENU_REPEAT			40022
#define WINAMP_MENU_SHUFFLE			40023

#define WINAMP_MV_PE_HLT_UP			105825
#define WINAMP_MV_PE_HLT_DOWN		105826

#define WINAMP_DEL_FRM_PE			66570
#define WINAMP_MV_PE_ITEM_DOWN		105752
#define WINAMP_MV_PE_ITEM_UP		105751

#define WINAMP_PE_SORT_TITLE		40209
#define WINAMP_PE_SORT_FNAME		40210
#define WINAMP_PE_SORT_FNAME_PATH	40211

#define WA_STATE_PLAY	100
#define WA_STATE_PAUSE	101
#define WA_STATE_STOP	102

//PE Stuff
#define IPC_PE_GETCURINDEX			100 
#define IPC_PE_GETINDEXTOTAL		101 
#define IPC_PE_GETINDEXINFO			102 
#define IPC_PE_GETINDEXINFORESULT	103 
#define IPC_PE_DELETEINDEX			104 
#define IPC_PE_SWAPINDEX			105 
#define IPC_PE_INSERTFILENAME		106 
#define IPC_PE_GETDIRTY				107 
#define IPC_PE_SETCLEAN				108 

class CWinampController : public PlayerController
{
public:
	CWinampController();
	virtual ~CWinampController();

	BOOL	Init(BOOL bStartWinamp);

	HWND	GetWinampWnd(){return m_hWndWinamp;}
	HWND    GetPEWnd(){return m_hWndWinampPE;}


	int GetPlayingTime();
	long GetCurrentSongLength();
	QString GetCurrentSongMetadata(QString);
	char* GetCurrentSongFileName();
	long GetPlayListPos();
	char* GetWinampDir();
	char* GetSongFileName(long PlayListPos);
	
	
	/*
	

	//Main Window - Operation
	BOOL	Play(char* strFile = "", BOOL bKeepList = FALSE);
	BOOL	Enqueue(char* strFile);
	void	Stop();
	void	Pause();
	void	FastForward();
	void	NextTrack();
	void	PrevTrack();
	void	Rewind();
	void	FastFwd5Secs();
	void	Rewind5Secs();
	void	CloseWinamp();
	void	RestartWinamp();
	void	IncreaseVolume();
	void	DecreaseVolume();

	BOOL	GetShuffleStatus();
	void	SetShuffleStatus(BOOL newVal);
	BOOL	GetRepeatStatus();
	void	SetRepeatStatus(BOOL newVal);
	HMENU	GetOptionMenu();
	long	GetVersion();
	long	GetStatus();
	void	SetStatus(long newVal);
	long	GetCurrentPos();
	void	SetCurrentPos(long newVal);
	void	SetVolume(long newVal);
	void	SetPanning(long PanIndex);
	void	ChangeDirectory(char* NewDir);
	char* GetSkinName();
	void	SetSkinName(char* Name);
	void	StartPlugIn(char* PlugInName);
	void	ToggleShufflePlay();
	void	ToggleRepeatPlay();
	void	SongChange();
	
	BOOL	GetWinampVisible();
	void	SetWinampVisible(BOOL newVal);

	//Song Information
	long	GetSampleRate();
	long	GetBitrate();
	long	GetChannels();
	
	char* GetSongTitle(long PlayListPos);
    char* GetSongMetaData(char* MetaData, long PlayListPos);
	
    char* GetCurrentSongTitle();
	

	//Playlist Functions
	void	ClearPlaylist();
	
	void	SetPlayListPos(long newVal);
	long	GetPlayListCount();
	void	DeletePEItem(int Pos);
	void	DeletePECurHighlight();
	void	MovePEHighlight(int Pos);
	bool	MovePEItem(int curPos, int NewPos);
	BOOL	GetPlaylistVisible();
	void	SetPlaylistVisible(BOOL newVal);
	void	JumpToTrack(unsigned int Number);
	void	SortByName(){SendMessage(m_hWndWinampPE, WM_COMMAND, WINAMP_PE_SORT_TITLE,0);}
	void	SortByFName(){SendMessage(m_hWndWinampPE, WM_COMMAND, WINAMP_PE_SORT_FNAME,0);}
	void	SortByFNamePath(){SendMessage(m_hWndWinampPE, WM_COMMAND, WINAMP_PE_SORT_FNAME_PATH,0);}
	void	PLInsertTrack(char* strFile, int nPos);
	BOOL	IsPEWindowWinshade();
	
	// Equaliser Functions
	long	GetEqPosition(long Band);
	void	SetEqPosition(long Band, long newVal);
	long	GetPreAmpPosition();
	void	SetPreAmpPosition(long newVal);
	BOOL	GetEqualizerVisible();
	void	SetEqualizerVisible(BOOL newVal);
	BOOL	GetEqEnabled();
	void	SetEqEnabled(BOOL newVal);
	BOOL	GetAutoloadEnabled();
	void	SetAutoloadEnabled(BOOL newVal);

	//Minibrowser stuff
	BOOL	GetMiniBrowserVisible();
	void	SetMiniBrowserVisible(BOOL newVal);
	void	SetMiniBrowserURL(char* strUrl, BOOL bForceOpen);
	BOOL  IsInetAvailable();

	int		GetTrackLength();
	
	char* GetCurrTrackPath();
	int		GetWinampState();
	char* GetWinampDir();

	BOOL	IsPlaying();
	BOOL	IsPaused();
	void	SetVolume(int nValue);
	int     GetVolume();
	char* GetFileNameFromPos(int nPos);
*/
protected:
	BOOL	WindowsReady();
	void	SendCommand(long command);
	
	HWND	m_hWndWinamp;
	HWND	m_hWndWinampEQ;
	HWND	m_hWndWinampPE;
	HWND	m_hWndWinampMB;
	int		m_nVolume;
	char*	m_strVersionString;		// Winamp Version String
	char*	m_strWinampDir;			// Winamp Directory
};
